#!/usr/bin/env python
# coding: utf-8

# ### Supply and Demand Levels Forecasting Based on Returns Volatility
# #### Leandro Guerra - Outspoken Market
# Get the full paper at:
#  https://www.outspokenmarket.com/diurnalis.html 

# In[1]:


# Imports

import pandas as pd
import numpy as np
import yfinance as yf
from plotly.subplots import make_subplots
import plotly.graph_objects as go

get_ipython().run_line_magic("matplotlib", "inline")
import warnings
warnings.filterwarnings("ignore")


# # Data preparation

# In[2]:


# Get the data

ticker1 = "^GSPC" 
df1 = yf.download(ticker1, "2012-01-01", "2022-12-31")
df1["Returns"] = df1["Adj Close"].pct_change(1)
df1["Adj Low"] = df1["Low"] - (df1["Close"]-df1["Adj Close"])
df1["Adj High"] = df1["High"] - (df1["Close"]-df1["Adj Close"])
df1["Adj Open"] = df1["Open"] - (df1["Close"]-df1["Adj Close"])
df1["Target"] = df1["Returns"].shift(-1)
df1.head()


# In[3]:


vol_p1 = 20
df1["Vol"] = np.round(df1["Returns"].rolling(vol_p1).std()*np.sqrt(252), 4)


# - Multiplying daily volatility by the square root of 252 gives annualized volatility;
# - If you divide the annualized volatility by the square root of:
#  - 12, it gives the monthly volatility
#  - 52, it gives the weekly volatility

# # Annual S&D Volatility Zones

# In[4]:


# For the Annualized Volatility Chart

fig = make_subplots(rows = 1, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.08)

fig.add_trace(go.Scatter(x = df1.index, y = df1["Vol"]*100
                         , name = "Vol", line = dict(color = "blue"))
              , row = 1, col = 1)


fig.update_layout(height = 600, width = 800
                  , title_text = "Annualized Volatility - " + ticker1 + " - www.outspokenmarket.com"
                  , font_color = "blue"
                  , title_font_color = "black"
                  , xaxis_title = "Years"
                  , yaxis_title = "Volatility (%)"
                  , legend_title = "Vol"
                  , font = dict(size = 15, color = "Black")
                 )
fig.update_layout(hovermode = "x")

# Code to exclude empty dates from the chart
dt_all = pd.date_range(start = df1.index[0]
                       , end = df1.index[-1]
                       , freq = "D")
dt_all_py = [d.to_pydatetime() for d in dt_all]
dt_obs_py = [d.to_pydatetime() for d in df1.index]

dt_breaks = [d for d in dt_all_py if d not in dt_obs_py]

fig.update_xaxes(
    rangebreaks = [dict(values = dt_breaks)]
)

fig.show()


# - A volatility of 15 represents a change, with a 68% probability, of a +-15% movement for the asset in 1 year
#  - This assumes that the returns are normally distributed, which we know is not the case...
#  - But it's a close enough approximation.

# In[5]:


year = "2021"

Upper_Band_12m1d =  df1[year]["Vol"][-1]*df1[year]["Adj Close"][-1] + df1[year]["Adj Close"][-1]
Lower_Band_12m1d =  df1[year]["Adj Close"][-1] - df1[year]["Vol"][-1]*df1[year]["Adj Close"][-1]

Upper_Band_12m2d =  2*df1[year]["Vol"][-1]*df1[year]["Adj Close"][-1] + df1[year]["Adj Close"][-1]
Lower_Band_12m2d =  df1[year]["Adj Close"][-1] - 2*df1[year]["Vol"][-1]*df1[year]["Adj Close"][-1]

# Annual S&D Volatility Zones chart

fig = make_subplots(rows = 1, cols = 1,
                    shared_xaxes = True,
                    vertical_spacing = 0.08)

fig.add_trace(go.Candlestick(x = df1[str(int(year)+1)].index
                             , open = df1[str(int(year)+1)]["Adj Open"], high = df1[str(int(year)+1)]["Adj High"]
                             , low = df1[str(int(year)+1)]["Adj Low"], close = df1[str(int(year)+1)]["Adj Close"]
                             , name = "Candle"
                             , increasing_line_color = "black", decreasing_line_color = "red")
              , row = 1, col = 1
             )

fig.add_hline(y = Upper_Band_12m1d, line_width = 1, line_dash = "dash", line_color = "green")
fig.add_hline(y = Lower_Band_12m1d, line_width = 1, line_dash = "dash", line_color = "red")

fig.add_hline(y = Upper_Band_12m2d, line_width = 1, line_dash = "dash", line_color = "green")
fig.add_hline(y = Lower_Band_12m2d, line_width = 1, line_dash = "dash", line_color = "red")


fig.update_layout(height = 600, width = 800
                  , title_text = "Annual S&D Volatility Zones: " + str(int(year)+1) + " " + ticker1
                  , font_color = "blue"
                  , title_font_color = "black"
                  , xaxis_title = "Years"
                  , yaxis_title = "Close"
                  , legend_title = "Vol"
                  , font = dict(size = 15, color = "Black")
                  , xaxis_rangeslider_visible = False
                 )
fig.update_layout(hovermode = "x")

# Code to exclude empty dates from the chart
dt_all = pd.date_range(start = df1.index[0]
                       , end = df1.index[-1]
                       , freq = "D")
dt_all_py = [d.to_pydatetime() for d in dt_all]
dt_obs_py = [d.to_pydatetime() for d in df1.index]

dt_breaks = [d for d in dt_all_py if d not in dt_obs_py]

fig.update_xaxes(
    rangebreaks = [dict(values = dt_breaks)]
)


fig.show()

